@@ -1,6 +1,6 @@
-upload_adls_file_internal <- function(filesystem, src, dest, blocksize=2^24, lease=NULL)
+upload_adls_file_internal <- function(filesystem, src, dest, blocksize=2^24, lease=NULL, put_md5=FALSE)
 {
-    src <- normalize_src(src)
+    src <- normalize_src(src, put_md5)
     on.exit(close(src$con))
 
     headers <- list()
@@ -22,8 +22,10 @@ upload_adls_file_internal <- function(filesystem, src, dest, blocksize=2^24, lea
             break
 
         opts <- list(action="append", position=sprintf("%.0f", pos))
-        headers <- list(`content-length`=sprintf("%.0f", thisblock))
-
+        headers <- list(
+            `content-length`=sprintf("%.0f", thisblock),
+            `content-md5`=encode_md5(body)
+        )
         do_container_op(filesystem, dest, headers=headers, body=body, options=opts, progress=bar$update(),
                         http_verb="PATCH")
 
@@ -34,45 +36,27 @@ upload_adls_file_internal <- function(filesystem, src, dest, blocksize=2^24, lea
     bar$close()
 
     # flush contents
+    headers <- list(`content-type`=src$content_type)
+    if(!is.null(src$md5))
+        headers$`x-ms-content-md5` <- src$md5
     do_container_op(filesystem, dest,
                     options=list(action="flush", position=sprintf("%.0f", pos)),
-                    headers=list(`content-type`=src$content_type),
+                    headers=headers,
                     http_verb="PATCH")
     invisible(NULL)
 }
 
 
-download_adls_file_internal <- function(filesystem, src, dest, blocksize=2^24, overwrite=FALSE)
+download_adls_file_internal <- function(filesystem, src, dest, blocksize=2^24, overwrite=FALSE, check_md5=FALSE)
 {
-    file_dest <- is.character(dest)
-    null_dest <- is.null(dest)
-    conn_dest <- inherits(dest, "rawConnection")
-
-    if(!file_dest && !null_dest && !conn_dest)
-        stop("Unrecognised dest argument", call.=FALSE)
-
     headers <- list()
-    if(file_dest)
-    {
-        if(!overwrite && file.exists(dest))
-            stop("Destination file exists and overwrite is FALSE", call.=FALSE)
-        if(!dir.exists(dirname(dest)))
-            dir.create(dirname(dest), recursive=TRUE)
-        dest <- file(dest, "w+b")
-        on.exit(close(dest))
-    }
-    if(null_dest)
-    {
-        dest <- rawConnection(raw(0), "w+b")
-        on.exit(seek(dest, 0))
-    }
-    if(conn_dest)
-        on.exit(seek(dest, 0))
+    dest <- init_download_dest(dest, overwrite)
+    on.exit(dispose_download_dest(dest))
 
-    # get file size (for progress bar)
-    res <- do_container_op(filesystem, src, headers=headers, http_verb="HEAD", http_status_handler="pass")
-    httr::stop_for_status(res, storage_error_message(res))
-    size <- as.numeric(httr::headers(res)[["Content-Length"]])
+    # get file size (for progress bar) and MD5 hash
+    props <- get_storage_properties(filesystem, src)
+    size <- as.numeric(props[["content-length"]])
+    src_md5 <- props[["content-md5"]]
 
     bar <- storage_progress_bar$new(size, "down")
     offset <- 0
@@ -89,5 +73,7 @@ download_adls_file_internal <- function(filesystem, src, dest, blocksize=2^24, o
     }
 
     bar$close()
-    if(null_dest) rawConnectionValue(dest) else invisible(NULL)
+    if(check_md5)
+        do_md5_check(dest, src_md5)
+    if(inherits(dest, "null_dest")) rawConnectionValue(dest) else invisible(NULL)
 }
