@@ -14,7 +14,9 @@ sign_request <- function(endpoint, ...)
 
 sign_request.default <- function(endpoint, verb, url, headers, api, ...)
 {
-    acct_name <- sub("\\..+$", "", url$host)
+    host_url <- httr::parse_url(endpoint$url)
+    acct_name <- if(host_url$path == "") sub("\\..+$", "", host_url$host) else host_url$path
+
     resource <- paste0("/", acct_name, "/", url$path) # don't use file.path because it strips trailing / on Windows
     # sanity check
     resource <- gsub("//", "/", resource)
